#include "YahooPacket.H"
#include "Logger.H"

extern Logger logger;

int YahooPacket::_headerSize=16;

YahooPacket::YahooPacket(yhPacketType ptype, unsigned int flag,
                const string ident, unsigned int ver) {
	_pkt=new char[_headerSize];
	memsetIdent(ident);
	memsetVer(ver);
	memsetType(ptype);
	memsetFlag(flag);
	memsetLen(0);
}

YahooPacket::~YahooPacket() {
	if (_pkt) {
		delete[](_pkt);
	}
}

int YahooPacket::setType(yhPacketType ptype, unsigned int flag) {
	memsetType(ptype);
	memsetFlag(flag);
	return 0;
}

int YahooPacket::setPayload(const char *buf1, int len1,
			const char *buf2, int len2, const char *buf3, int len3) {
	unsigned int
		len;
	char
		*payload;

	len=len1+(len2?1:0)+len2+(len3?1:0)+len3;
	if ((_pkt=(char *)realloc(_pkt, _headerSize+len))==NULL) {
		return -1;
	}
	payload=_pkt+_headerSize;
	memcpy(payload, buf1, len1);
	if (len2) {
		payload[len1]=YH_ODELIMITER;
		memcpy((payload+len1+1), buf2, len2);
		if (len3) {
			payload[len1+1+len2]=YH_ODELIMITER;
			memcpy((payload+len1+len2+2), buf3, len3);
		}
	}
	memsetLen(len);
	return 0;
}

int YahooPacket::setInbound(const char *hdr) {
	memcpy(_pkt,hdr,_headerSize);
	return 0;
}

string YahooPacket::getIdent() const {
	char
		str[5];

	memcpy(str,_pkt,4);
	str[4]=0;
	return (str);
}

unsigned int YahooPacket::getVer() const {
	unsigned int
		ver;

	memcpy(&ver,(_pkt+4),sizeof(ver));
	ver=ntohl(ver);
	return (ver);
}

yhPacketType YahooPacket::getType() const {
	unsigned int
		type;

	memcpy(&type,(_pkt+8),sizeof(type));
	type=ntohl(type);
	return (yhPacketType)(type);
}

unsigned int YahooPacket::getFlag() const {
	unsigned short
		flag;

	memcpy(&flag,(_pkt+12),sizeof(flag));
	flag=ntohs(flag);
	return (flag);
}

unsigned int YahooPacket::getLen() const {
	unsigned short
		len;

	memcpy(&len,(_pkt+14),sizeof(len));
	len=ntohs(len);
	return (len);
}

int YahooPacket::memsetIdent(const string ident) {
	if (ident.length() >= 4) {
		memcpy(_pkt,ident.c_str(),4);
		return (ident.length() -4);
	}
	return -1;	
}

int YahooPacket::memsetVer(unsigned int ver) {
	unsigned int
		*ptr;

	ptr = (unsigned int *) (_pkt+4);
	*ptr = htonl(ver);
	return 0;
}

int YahooPacket::memsetType(unsigned int type) {
	unsigned int
		*ptr;

	ptr = (unsigned int *) (_pkt+8);
	*ptr = htonl(type);
	return 0;
}

int YahooPacket::memsetFlag(unsigned int flag) {
	unsigned short
		*ptr;

	ptr = (unsigned short *) (_pkt+12);
	*ptr = htons(flag);
	return 0;
}

int YahooPacket::memsetLen(unsigned int len) {
	unsigned short
		*ptr;

	ptr = (unsigned short *) (_pkt+14);
	*ptr = htons(len);
	return 0;
}
